<?php
/*
Plugin Name: I18N Base
Description: Internationalization based on slug/URL names (e.g. index, index_de, index_fr)
Version: 1.8
Author: Martin Vlcek
Author URI: http://mvlcek.bplaced.net

Public functions:
  return_i18n_default_language()
      returns the default language - the language of pages without language suffix
  return_i18n_languages()
      returns an array of user requested languages with the best first, e.g. ( 'de', 'fr', 'en' )
  return_i18n_page_data($slug)
      returns the xml data for the best fitting language version of the $slug
  return_i18n_lang_url($language=null)
      returns the URL to the current page in the given $language (if null, the default language is used)
      (you should use htmlspecialchars when outputting in a href to convert & to &amp;)
  return_i18n_setlang_url($language)
      returns the URL to the current page which also sets the preferred $language. If the current URL does not have a 
      parameter lang then this causes the page to be displayed in the given $language (if it exists).
      (you should use htmlspecialchars when outputting in a href to convert & to &amp;)
  find_i18n_url($slug,$parent,$language,$type='full')
      returns the URL to the page identified by $slug/$parent in the given $language (see also core function find_url)
      (you should use htmlspecialchars when outputting in a href to convert & to &amp;)
  return_i18n_component($slug)
      returns the component content (unprocessed)

Display functions:
  get_i18n_content($slug)
      outputs the best fitting language content of the $slug. Returns true, if content found.
  get_i18n_component($id, $param1, ...)
      outputs the (localized) component. Returns true, if component found.
      Optionally parameters can be passed. They are available in the component as global array $args.
  get_i18n_link($slug)
      outputs a link to the given page in the best language

Functions to call for other plugins:
  i18n_init()
      loads the correct language version for the current page
*/

# get correct id for plugin
$thisfile = basename(__FILE__, ".php");
$i18n_initialized = false;
$i18n_languages = null;
$i18n_settings = null;

define('I18N_DEFAULT_LANGUAGE', 'en');
define('I18N_SETTINGS_FILE', 'i18n_settings.xml');
define('I18N_LANGUAGE_PARAM', 'lang');            # language parameter in URL, e.g. "...?lang=de"
define('I18N_SET_LANGUAGE_PARAM', 'setlang');     # parameter to set current language via GET/POST, e.g. "...?setlang=de"
define('I18N_LANGUAGE_KEY', 'language');          # session key for language, if language was set 

// properties
define('I18N_PROP_DEFAULT_LANGUAGE', 'default_language');
define('I18N_PROP_URLS_TO_IGNORE', 'urls-to-ignore');
define('I18N_PROP_PAGES_VIEW', 'pages-view');

# register plugin
register_plugin(
	$thisfile, 
	'I18N Base', 	
	'1.8', 		
	'Martin Vlcek',
	'http://mvlcek.bplaced.net', 
	'Internationalize content based on slug/URL names',
	'pages',
	'i18n_pages'  
);

require_once(GSPLUGINPATH.'i18n_common/common.php');
i18n_load_texts('i18n_base', 'en');

# activate filter
add_action('index-pretemplate', 'i18n_init');
add_action('edit-extras', 'i18n_base_edit'); 
add_action('pages-sidebar', 'createSideMenu', array($thisfile, i18n_r('i18n_base/PAGES')));
add_action('admin-pre-header', 'i18n_redirect_pages'); // 3.1+ only

# functions

// load texts based on frontend/admin languages
function i18n_load_texts($plugin, $default_language="en_US") {
  global $LANG, $language;
  if (basename($_SERVER['PHP_SELF']) == 'index.php') {
    // frontend language with I18N plugin is always two characters long
    i18n_merge($plugin, $language) ||
    i18n_merge($plugin, return_i18n_default_language()) || 
    i18n_merge($plugin, 'en');
  } else {
    i18n_merge($plugin, $LANG) ||
    (strlen($LANG) > 2 && i18n_merge($plugin, substr($LANG,0,2))) ||
    i18n_merge($plugin, 'en_US') ||
    i18n_merge($plugin, 'en');
  }
}

function return_i18n_default_language() {
  return i18n_get_property(I18N_PROP_DEFAULT_LANGUAGE, I18N_DEFAULT_LANGUAGE);
}

function return_i18n_languages() {
  global $i18n_languages;
  if (!$i18n_languages) {
    $i18n_languages = array();
    if (isset($_GET[I18N_LANGUAGE_PARAM])) {
      $i18n_languages[] = $_GET[I18N_LANGUAGE_PARAM];
    }
    if (isset($_SESSION[I18N_LANGUAGE_KEY]) && !in_array($_SESSION[I18N_LANGUAGE_KEY],$i18n_languages)) {
      $i18n_languages[] = $_SESSION[I18N_LANGUAGE_KEY];
    }
    $httplanguages = explode(",", @$_SERVER['HTTP_ACCEPT_LANGUAGE']);
    foreach ($httplanguages as $language) {
      $language = substr($language,0,2);
      if (!in_array($language,$i18n_languages)) $i18n_languages[] = $language;
    }
    $deflang = return_i18n_default_language();
    if (!in_array($deflang, $i18n_languages)) $i18n_languages[] = $deflang;
  }
  return $i18n_languages;
}

function i18n_init() {
  global $i18n_initialized;
  if ($i18n_initialized) return;
  @session_start();
  if (isset($_GET[I18N_SET_LANGUAGE_PARAM])) {
    $_SESSION[I18N_LANGUAGE_KEY] = $_GET[I18N_SET_LANGUAGE_PARAM];
  }
  $languages = return_i18n_languages();
  $i18n_initialized = true;
  foreach ($languages as $language) {
    if (i18n_load($language)) return;
  }
}

function i18n_load($lang) {
  global $language, $data_index_orig;
  global $data_index, $id, $title, $date, $metak, $metad, $content, $parent, $template_file, $private;
  global $PERMALINK, $PERMALINK_ORIG;
  $deflang = return_i18n_default_language();
  if ($lang == $deflang) {
    $language = $deflang;
  } else { 
    // load language
    $file = GSDATAPAGESPATH . $id . '_' . $lang .'.xml';
    if (file_exists($file)) {
      $data = getXML($file);
      $title = $data->title;
      $date = $data->pubDate;
      $metak = $data->meta;
      $metad = $data->metad;
      // $url = $data->url;  // URL stays the same!
      $content = $data->content;
      $parent = $data->parent;
      // $template_file = $data->template;  // template stays the same
      $private = $data->private;
      // language
      $language = $lang;
      $data_index_orig = $data_index;
      $data_index = $data;
    } else {
      return false;
    }
  }
  if (@$PERMALINK) {
    $PERMALINK_ORIG = $PERMALINK;
    $PERMALINK = str_replace('%language%', @$_GET['lang']?$_GET['lang']:$language, $PERMALINK);
  }
  return true;
}

function return_i18n_page_data($slug) {
  $languages = return_i18n_languages();
  $deflang = return_i18n_default_language();
  foreach ($languages as $lang) {
    $file = GSDATAPAGESPATH . $slug . ($lang == $deflang ? '' : '_' . $lang) .'.xml';
    if (file_exists($file)) {
      return getXML($file);
    }
  }
  return null;
}

function get_i18n_content($slug, $force=false) {
  $data = null;
  if ($force) {
    $file = GSDATAPAGESPATH . $slug . '.xml';
    if (file_exists($file)) $data = getXML($file);
  } else {
    $data = return_i18n_page_data($slug);
  }
  if ($data) {
    $content = $data->content;
    $content = stripslashes(htmlspecialchars_decode($content, ENT_QUOTES));
    $content = exec_filter('content',$content);
    echo $content;
    return true;
  } else {
    return false;
  }
}

function return_i18n_component($id) {
	if (file_exists(GSDATAOTHERPATH.'components.xml')) {
    $deflang = return_i18n_default_language();
		$data = getXML(GSDATAOTHERPATH.'components.xml');
		$components = $data->item;
  	if (count($components) != 0) {
      $languages = return_i18n_languages();
      $isV3 = function_exists('get_site_version'); 
      $sep = $isV3 ? '_' : '';
      foreach ($languages as $lang) {
        $slug = $id . ($lang == $deflang ? '' : $sep . $lang);
			  foreach ($components as $component) {
				  if ($slug == $component->slug) { 
					  return stripslashes(htmlspecialchars_decode($component->value, ENT_QUOTES)); 
				  }
			  }
      }
		}
	}
  return null;
}

function get_i18n_component($id, $param1=null, $param2=null) {
  global $args;
  $component = return_i18n_component($id);
  if (isset($args)) $saved_args = $args;
  if (func_num_args() > 1) { $args = func_get_args(); array_shift($args); }   
  if ($component) eval("?>" . $component . "<?php ");
  if (isset($saved_args)) $args = $saved_args; else unset($args); 
  return $component ? true : false;
}

function i18n_get_property($name, $default = null) {
  global $i18n_settings;
  if ($i18n_settings == null) {
    $i18n_settings = array();
    if (file_exists(GSDATAOTHERPATH . I18N_SETTINGS_FILE)) {
      $data = getXML(GSDATAOTHERPATH . I18N_SETTINGS_FILE);
      foreach ($data->children() as $child) $i18n_settings[$child->getName()] = (string) $child;
    }
  }
  return isset($i18n_settings[$name]) ? $i18n_settings[$name] : $default;
}

function i18n_set_property($name, $value) {
  i18n_set_properties(array($name => $value));
}

function i18n_set_properties($properties) {
  global $i18n_settings;
  i18n_get_property(null);
  foreach ($properties as $key => $value) $i18n_settings[$key] = (string) $value;
	$data = @new SimpleXMLExtended('<?xml version="1.0" encoding="UTF-8"?><settings></settings>');
  foreach ($i18n_settings as $key => $value) {
    if ($value != null) {
      $propdata = $data->addChild($key);
      $propdata->addCData((string) $value);
    }
  }
	XMLsave($data, GSDATAOTHERPATH . I18N_SETTINGS_FILE);
}

function i18n_add_urls_to_ignore($pattern) {
  $patterns = preg_split('/\|/', i18n_get_property(I18N_PROP_URLS_TO_IGNORE, ''));
  if (!in_array($pattern, $patterns)) $patterns[] = $pattern;
  i18n_set_property(I18N_PROP_URLS_TO_IGNORE, implode('|', $patterns));
}

function i18n_remove_urls_to_ignore($pattern) {
  $patterns = preg_split('/\|/', i18n_get_property(I18N_PROP_URLS_TO_IGNORE, ''));
  $patterns = array_diff($patterns, array($pattern));
  i18n_set_property(I18N_PROP_URLS_TO_IGNORE, implode('|', $patterns));
}

function i18n_pages() {
  include(GSPLUGINPATH.'i18n_base/pages.php');
}

function i18n_base_edit() {
  include(GSPLUGINPATH.'i18n_base/editextras.php');
}

function find_i18n_url($slug, $slugparent, $language=null, $type='full') {
  global $url, $parent, $PERMALINK, $PERMALINK_ORIG;
  if (!$slug) {
    $slug = @$url;
    $slugparent = @$parent;
  }
  if ($language && @strpos(@$PERMALINK_ORIG,'%language%') !== false) {
    $PERMALINK_SAVED = $PERMALINK;
    $PERMALINK = str_replace('%language%', $language, $PERMALINK_ORIG);
    $u = find_url($slug, $slugparent, $type);
    $PERMALINK = $PERMALINK_SAVED;
  } else { 
    $u = find_url($slug, $slugparent, $type);
    if ($language) {
      $u .= (strpos($u,'?') !== false ? '&' : '?') . 'lang=' . $language;
    }
  }
  return $u;
}

function i18n_current_url_with_param($name, $value) {
  $pos = strpos($_SERVER["REQUEST_URI"],'?');
  if ($pos !== false) {
    $url = substr($_SERVER["REQUEST_URI"],0,$pos+1);
    $params = preg_split('/&/', substr($_SERVER["REQUEST_URI"], $pos+1));
    foreach ($params as $param) {
      if (substr($param,0,strlen($name)+1) != $name.'=') $url .= $param.'&';
    }
  } else {
    $url = $_SERVER["REQUEST_URI"].'?';
  }
  if ($value != null) $url .= $name.'='.urlencode($value); else $url = substr($url,0,strlen($url)-1);
  return $url;
}

function return_i18n_lang_url($language=null) {
  if (!$language) $language = return_i18n_default_language();
  if (@strpos(@$PERMALINK_ORIG,'%language%') !== false) {
    return find_i18n_url(null,null,$language); 
  } else {
    return i18n_current_url_with_param('lang',$language);
  }
}

function return_i18n_setlang_url($language) {
  global $PERMALINK_ORIG;
  if (@strpos(@$PERMALINK_ORIG,'%language%') !== false) {
    return find_i18n_url(null,null,$language); // no setlang parameter supported for URLs with language
  } else {
    return i18n_current_url_with_param('setlang',$language);
  }
}

function get_i18n_link($slug) {
  $data = return_i18n_page_data($slug);
  if (!$data) return false;
  echo '<a href="'.find_url($slug,(string) $data->parent).'">'.stripslashes((string) $data->title).'</a>';
  return true;
}

function i18n_redirect_pages() {
  if (basename($_SERVER['PHP_SELF']) == 'pages.php') {
    header('Location: /admin/load.php?id=i18n_base');
    exit(0);
  }
}
